# Copyright 2016 Timo Kankare <timo.kankare@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

"""Backends to implement mail box specific functionality, like IMAP and POP3."""

from collections import namedtuple
import json
import re

from Mailnag.backends.imap import IMAPMailboxBackend
from Mailnag.backends.pop3 import POP3MailboxBackend
from Mailnag.backends.local import MBoxBackend, MaildirBackend
from Mailnag.common.utils import splitstr


def _str_to_folders(folders_str):
	if re.match(r'^\[.*\]$', folders_str):
		folders	= json.loads(folders_str)
	else:
		folders	= splitstr(folders_str, ',')
	return folders


def _folders_to_str(folders):
	return json.dumps(folders, ensure_ascii=False)


def _str_to_bool(string):
	return bool(int(string))


def _bool_to_str(b):
	return str(int(b))


Param = namedtuple('Param',
    ['param_name', 'option_name', 'from_str', 'to_str', 'default_value']
)
Backend = namedtuple('Backend', ['backend_class', 'params'])

_backends = {
	'imap' : Backend(IMAPMailboxBackend, [
				Param('user', 'user', str, str, ''),
				Param('password', 'password', str, str, ''),
				Param('server', 'server', str, str, ''),
				Param('port', 'port', str, str, ''),
				Param('ssl', 'ssl', _str_to_bool, _bool_to_str, True),
				Param('imap', 'imap', _str_to_bool, _bool_to_str, True),
				Param('idle', 'idle', _str_to_bool, _bool_to_str, True),
				Param('folders', 'folder', _str_to_folders, _folders_to_str, []),
			 ]),
	'pop3' : Backend(POP3MailboxBackend, [
				Param('user', 'user', str, str, ''),
				Param('password', 'password', str, str, ''),
				Param('server', 'server', str, str, ''),
				Param('port', 'port', str, str, ''),
				Param('ssl', 'ssl', _str_to_bool, _bool_to_str, True),
				Param('imap', 'imap', _str_to_bool, _bool_to_str, False),
				Param('idle', 'idle', _str_to_bool, _bool_to_str, False),
			 ]),
	'mbox' : Backend(MBoxBackend, [
				Param('path', 'path', str, str, ''),
			 ]),
	'maildir' : Backend(MaildirBackend, [
				Param('path', 'path', str, str, ''),
				Param('folders', 'folder', _str_to_folders, _folders_to_str, []),
			]),
}


def create_backend(mailbox_type, **kw):
	"""Create mailbox backend of specified type and parameters."""
	return _backends[mailbox_type].backend_class(**kw)


def get_mailbox_parameter_specs(mailbox_type):
	"""Returns mailbox backend specific parameter specification.
	The specification is a list objects which have atributes:
	* param_name - the name of argument in which parameter value is given to
				   backend constructor
	* option_name - the name used in configuration
	* from_str - the function to convert string to parameter specific type
	* default_value - the value used when the option is not found
	"""
	return _backends[mailbox_type].params




# Copyright 2013 - 2020 Patrick Ulbrich <zulu99@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

import gi
gi.require_version('Gst', '1.0')

import os
import threading
from gi.repository import Gst
from Mailnag.common.plugins import Plugin, HookTypes
from Mailnag.common.utils import get_data_file
from Mailnag.common.i18n import _


plugin_defaults = { 'soundfile' : 'mailnag.ogg' }


class SoundPlugin(Plugin):
	def __init__(self):
		self._mails_added_hook = None

	
	def enable(self):
		def mails_added_hook(new_mails, all_mails):
			config = self.get_config()
			gstplay(get_data_file(config['soundfile']))
		
		self._mails_added_hook = mails_added_hook
		
		controller = self.get_mailnag_controller()
		hooks = controller.get_hooks()
		
		hooks.register_hook_func(HookTypes.MAILS_ADDED, 
			self._mails_added_hook)
		
	
	def disable(self):
		controller = self.get_mailnag_controller()
		hooks = controller.get_hooks()
		
		if self._mails_added_hook != None:
			hooks.unregister_hook_func(HookTypes.MAILS_ADDED,
				self._mails_added_hook)
			self._mails_added_hook = None

	
	def get_manifest(self):
		return (_("Sound Notifications"),
				_("Plays a sound when new mails arrive."),
				"2.0",
				"Patrick Ulbrich <zulu99@gmx.net>")


	def get_default_config(self):
		return plugin_defaults
	
	
	def has_config_ui(self):
		return False
	
	
	def get_config_ui(self):
		# TODO : Add ui to specify the path 
		# of a custom sound file.
		return None
	
	
	def load_ui_from_config(self, config_ui):
		pass
	
	
	def save_ui_to_config(self, config_ui):
		pass


class _GstPlayThread(threading.Thread):
	def __init__(self, ply):
		self.ply = ply
		threading.Thread.__init__(self)
	
	
	def run(self):
		def on_eos(bus, msg):
#			loggin.debug('EOS')
			self.ply.set_state(Gst.State.NULL)
			return True
		
		bus = self.ply.get_bus()
		bus.add_signal_watch()
		bus.connect('message::eos', on_eos)
		
		self.ply.set_state(Gst.State.PLAYING)


_gst_initialized = False

def gstplay(filename):
	global _gst_initialized
	if not _gst_initialized:
		Gst.init(None)
		_gst_initialized = True
		
	try:
		ply = Gst.ElementFactory.make("playbin", "player")
		ply.set_property("uri", "file://" + os.path.abspath(filename))
		pt = _GstPlayThread(ply)
		pt.start()
	except:
		pass



# Copyright 2011 - 2020 Patrick Ulbrich <zulu99@gmx.net>
# Copyright 2016 Thomas Haider <t.haider@deprecate.de>
# Copyright 2016, 2018 Timo Kankare <timo.kankare@iki.fi>
# Copyright 2011 Ralf Hersel <ralf.hersel@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

import logging
import hashlib
from Mailnag.backends import create_backend, get_mailbox_parameter_specs
from Mailnag.common.secretstore import SecretStore
from Mailnag.common.dist_cfg import PACKAGE_NAME

account_defaults = {
	'enabled'			: '0',
	'type'				: 'imap',
	'name'				: '',	
	'user'				: '',
	'password'			: '',
	'server'			: '',
	'port'				: '',
	'ssl'				: '1',
	'imap'				: '1',	
	'idle'				: '1',
	'folder'			: '[]'
}

#
# Account class
#
class Account:
	def __init__(self, mailbox_type = None, enabled = False, name = '', **kw):
		self._backend = None
		self.set_config(
			mailbox_type=mailbox_type,
			name=name,
			enabled=enabled,
			config=kw)


	def set_config(self, mailbox_type, enabled, name, config):
		"""Set accounts configuration."""
		self.enabled = enabled
		if mailbox_type:
			self.mailbox_type = mailbox_type
		elif 'imap' in config:
			self.mailbox_type = 'imap' if config.get('imap', True) else 'pop3'
		else:
		    self.mailbox_type = ''
		self.name = name
		self.user = config.get('user', '')
		self.password = config.get('password', '')
		self.oauth2string = config.get('oauth2string', '')
		self.server = config.get('server', '')
		self.port = config.get('port', '')
		self.ssl = config.get('ssl', True)
		self.imap = config.get('imap', True)
		self.idle = config.get('idle', False)
		self.folders = config.get('folders', [])
		self._rest_of_config = config
		if self._backend and self._backend.is_open():
			self._backend.close()
		self._backend = None


	def get_config(self):
		"""Return account's configuration as a dict."""
		config = {
			'enabled': self.enabled,
			'mailbox_type': self.mailbox_type,
			'name': self.name,
		}
		config.update(self._get_backend_config())
		return config


	def open(self):
		"""Open mailbox for the account."""
		self._get_backend().open()


	def close(self):
		"""Close mailbox for this account."""
		self._get_backend().close()


	# Indicates whether the account 
	# holds an active existing connection.
	def is_open(self):
		"""Returns true if the mailbox is opened."""
		return self._get_backend().is_open()


	def list_messages(self):
		"""Lists unseen messages from the mailbox for this account.
		Yields a set of tuples (folder, message).
		"""
		return self._get_backend().list_messages()


	def supports_notifications(self):
		"""Returns True if account supports notifications."""
		return self._get_backend().supports_notifications()


	def notify_next_change(self, callback=None, timeout=None):
		"""Asks mailbox to notify next change.
		Callback is called when new mail arrives or removed.
		This may raise an exception if mailbox does not support
		notifications.
		"""
		self._get_backend().notify_next_change(callback, timeout)


	def cancel_notifications(self):
		"""Cancels notifications.
		This may raise an exception if mailbox does not support
		notifications.
		"""
		self._get_backend().cancel_notifications()


	def request_server_folders(self):
		"""Requests folder names (list) from a server.
		Returns an empty list if mailbox does not support folders.
		"""
		return self._get_backend().request_folders()


	def supports_mark_as_seen(self):
		return self._get_backend().supports_mark_as_seen()


	def mark_as_seen(self, mails):
		self._get_backend().mark_as_seen(mails)
		
			
	def get_id(self):
		"""Returns unique id for the account."""
		# Assumption: The name of the account is unique.
		return str(hash(self.name))


	def _get_backend(self):
		if not self._backend:
			backend_config = self._get_backend_config()
			self._backend = create_backend(self.mailbox_type,
										   name=self.name,
										   **backend_config)
		return self._backend


	def _get_backend_config(self):
		config = {}
		imap_pop_config = {
			'user': self.user,
			'password': self.password,
			'oauth2string': self.oauth2string,
			'server': self.server,
			'port': self.port,
			'ssl': self.ssl,
			'imap': self.imap,
			'idle': self.idle,
			'folders': self.folders,
		}
		config.update(imap_pop_config)
		config.update(self._rest_of_config)
		return config


#
# AccountManager class
#
class AccountManager:
	def __init__(self):
		self._accounts = []
		self._removed = []
		self._secretstore = SecretStore.get_default()
		
		if self._secretstore == None:
			logging.warning("Failed to create secretstore - account passwords will be stored in plaintext config file.")

	
	def __len__(self):
		return len(self._accounts)
	
	
	def __iter__(self):
		for acc in self._accounts:
			yield acc

		
	def __contains__(self, item):
		return (item in self._accounts)
	
	
	def add(self, account):
		self._accounts.append(account)
	
	
	def remove(self, account):
		self._accounts.remove(account)
		self._removed.append(account)
	
	
	def clear(self):
		for acc in self._accounts:
			self._removed.append(acc)
		del self._accounts[:]
	
	
	def to_list(self):
		# Don't pass a ref to the internal accounts list.
		# (Accounts must be removed via the remove() method only.)
		return self._accounts[:]
	
	
	def load_from_cfg(self, cfg, enabled_only = False):
		del self._accounts[:]
		del self._removed[:]
		
		i = 1
		section_name = "account" + str(i)
		
		while cfg.has_section(section_name):
			enabled		= bool(int(	self._get_account_cfg(cfg, section_name, 'enabled')	))
			
			if (not enabled_only) or (enabled_only and enabled):
				if cfg.has_option(section_name, 'type'):
					mailbox_type = self._get_account_cfg(cfg, section_name, 'type')
					imap = (mailbox_type == 'imap')
				else:
					imap = bool(int(self._get_account_cfg(cfg, section_name, 'imap')))
					mailbox_type = 'imap' if imap else 'pop3'
				name = self._get_account_cfg(cfg, section_name, 'name')

				option_spec = get_mailbox_parameter_specs(mailbox_type)
				options = self._get_cfg_options(cfg, section_name, option_spec)

				# TODO: Getting a password from the secretstore is mailbox specific.
				#       Not every backend requires a password.
				user = options.get('user')
				server = options.get('server')
				if self._secretstore != None and user and server:
					password = self._secretstore.get(self._get_account_id(user, server, imap))
					if not password: password = ''
					options['password'] = password

				acc = Account(enabled=enabled,
							  name=name,
							  mailbox_type=mailbox_type,
							  **options)
				self._accounts.append(acc)

			i = i + 1
			section_name = "account" + str(i)
			

	def save_to_cfg(self, cfg):		
		# Remove all accounts from cfg
		i = 1
		section_name = "account" + str(i)
		while cfg.has_section(section_name):
			cfg.remove_section(section_name)
			i = i + 1
			section_name = "account" + str(i)
		
		# Delete secrets of removed accounts from the secretstore
		# (it's important to do this before adding accounts, 
		# in case multiple accounts with the same id exist).
		if self._secretstore != None:
			for acc in self._removed:
				self._secretstore.remove(self._get_account_id(acc.user, acc.server, acc.imap))
			
		del self._removed[:]
		
		# Add accounts
		i = 1
		for acc in self._accounts:
			if acc.oauth2string != '':
				logging.warning("Saving of OAuth2 based accounts is not supported. Account '%s' skipped." % acc.name)
				continue
				
			section_name = "account" + str(i)
			
			cfg.add_section(section_name)
			
			cfg.set(section_name, 'enabled', int(acc.enabled))
			cfg.set(section_name, 'type', acc.mailbox_type)
			cfg.set(section_name, 'name', acc.name)

			config = acc.get_config()
			option_spec = get_mailbox_parameter_specs(acc.mailbox_type)

			# TODO: Storing a password is mailbox specific.
			#       Not every backend requires a password.
			if self._secretstore != None:
				self._secretstore.set(self._get_account_id(acc.user, acc.server, acc.imap), acc.password,
				        f'{PACKAGE_NAME.capitalize()} password for account {acc.user}@{acc.server}')
				config['password'] = ''
                        
			self._set_cfg_options(cfg, section_name, config, option_spec)

			i = i + 1
	
		
	def _get_account_id(self, user, server, is_imap):
                # TODO : Introduce account.uuid when rewriting account and backend code
                return hashlib.md5((user + server + str(is_imap)).encode('utf-8')).hexdigest()
	
	
	def _get_account_cfg(self, cfg, section_name, option_name):
		if cfg.has_option(section_name, option_name):
			return cfg.get(section_name, option_name)
		else:
			return account_defaults[option_name]


	def _get_cfg_options(self, cfg, section_name, option_spec):
		options = {}
		for s in option_spec:
			options[s.param_name] = self._get_cfg_option(cfg,
														 section_name,
														 s.option_name,
														 s.from_str,
														 s.default_value)
		return options


	def _get_cfg_option(self, cfg, section_name, option_name, convert, default_value):
		if convert and cfg.has_option(section_name, option_name):
			value = convert(cfg.get(section_name, option_name))
		else:
			value = default_value
		return value


	def _set_cfg_options(self, cfg, section_name, options, option_spec):
		for s in option_spec:
			if s.to_str and s.param_name in options:
				value = s.to_str(options[s.param_name])
			else:
				value = s.default_value
			cfg.set(section_name, s.option_name, value)




# Copyright 2013 - 2020 Patrick Ulbrich <zulu99@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

import gi
gi.require_version('Gtk', '3.0')

from gi.repository import Gtk
from Mailnag.common.plugins import Plugin, HookTypes
from Mailnag.common.i18n import _

plugin_defaults = { 'filter_text' : 'newsletter, viagra' }


class SpamfilterPlugin(Plugin):
	def __init__(self):
		self._filter_mails_hook = None
		self._filter_list = None
	
	def enable(self):
		config = self.get_config()
		self._filter_list = config['filter_text'].replace('\n', '').split(',')
		
		def filter_mails_hook(mails):
			lst = []
			for m in mails:
				if not self._is_filtered(m):
					lst.append(m)
			return lst
		
		self._filter_mails_hook = filter_mails_hook
		
		controller = self.get_mailnag_controller()
		hooks = controller.get_hooks()
		
		hooks.register_hook_func(HookTypes.FILTER_MAILS, 
			self._filter_mails_hook)
		
	
	def disable(self):
		controller = self.get_mailnag_controller()
		hooks = controller.get_hooks()
		
		if self._filter_mails_hook != None:
			hooks.unregister_hook_func(HookTypes.FILTER_MAILS,
				self._filter_mails_hook)
			self._filter_mails_hook = None
		
		self._filter_list = None
	
	
	def get_manifest(self):
		return (_("Spam Filter"),
				_("Filters out unwanted mails."),
				"2.0",
				"Patrick Ulbrich <zulu99@gmx.net>")


	def get_default_config(self):
		return plugin_defaults
	
	
	def has_config_ui(self):
		return True
	
	
	def get_config_ui(self):
		box = Gtk.Box()
		box.set_spacing(12)
		box.set_orientation(Gtk.Orientation.VERTICAL)
		#box.set_size_request(100, -1)
		
		desc =  _('Mailnag will ignore mails containing at least one of \nthe following words in subject or sender.')
		
		label = Gtk.Label(desc)
		label.set_line_wrap(True)
		#label.set_size_request(100, -1);
		box.pack_start(label, False, False, 0)
		
		scrollwin = Gtk.ScrolledWindow()
		scrollwin.set_shadow_type(Gtk.ShadowType.IN)
		scrollwin.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
		scrollwin.set_size_request(-1, 60)
		
		txtbuffer = Gtk.TextBuffer()
		txtview = Gtk.TextView()
		txtview.set_buffer(txtbuffer)
		txtview.set_wrap_mode(Gtk.WrapMode.WORD)
		
		scrollwin.add(txtview)
		
		box.pack_start(scrollwin, True, True, 0)
		
		return box
	
	
	def load_ui_from_config(self, config_ui):
		config = self.get_config()
		txtview = config_ui.get_children()[1].get_child()
		txtview.get_buffer().set_text(config['filter_text'])
	
	
	def save_ui_to_config(self, config_ui):
		config = self.get_config()
		txtview = config_ui.get_children()[1].get_child()
		txtbuffer = txtview.get_buffer()
		start, end = txtbuffer.get_bounds()
		config['filter_text'] = txtbuffer.get_text(start, end, False)
	
	
	def _is_filtered(self, mail):
		is_filtered = False
		
		for f in self._filter_list:
			# remove CR and white space
			f = f.strip()			
			
			if len (f) == 0:
				continue
			
			f = f.lower()
			sender_name, sender_addr = mail.sender
			if (f in sender_name.lower()) or (f in sender_addr.lower()) \
				or (f in mail.subject.lower()):
				# sender or subject contains filter string
				is_filtered = True
				break
		
		return is_filtered



# Copyright 2013 - 2019 Patrick Ulbrich <zulu99@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

import gi
gi.require_version('Gtk', '3.0')

from gi.repository import Gtk
from Mailnag.common.i18n import _


class PluginDialog:
	def __init__(self, parent, plugin):
		self._plugin = plugin
		
		self._window = Gtk.Dialog(title = _('Plugin Configuration'), parent = parent, use_header_bar = True, \
			buttons = (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OK, Gtk.ResponseType.OK))
		
		self._window.set_default_response(Gtk.ResponseType.OK)
		self._window.set_default_size(480, 0)
		
		self._box = self._window.get_content_area()
		self._box.set_border_width(12)
		self._box.set_spacing(12)

		
	def run(self):
		widget = self._plugin.get_config_ui()
		
		if widget != None:
			self._box.pack_start(widget, True, True, 0)
			widget.show_all()
			self._plugin.load_ui_from_config(widget)
		
		res = self._window.run()
		
		if res == Gtk.ResponseType.OK:
			if widget != None:
				self._plugin.save_ui_to_config(widget)
		
		self._window.destroy()
		return res



# Copyright 2020 Patrick Ulbrich <zulu99@gmx.net>
# Copyright 2016 Timo Kankare <timo.kankare@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

"""Interface and base implementation for mailbox backends."""

from abc import ABCMeta, abstractmethod


class MailboxBackend(object, metaclass=ABCMeta):
	"""Interface for mailbox backends.

	Mailbox backend implements access to the specific type of mailbox.
	"""

	def __init__(self, **kw):
		"""Constructor should accept any kind of backend specific
		parameters.
		"""

	@abstractmethod
	def open(self):
		"""Opens the mailbox."""
		raise NotImplementedError

	@abstractmethod
	def close(self):
		"""Closes the mailbox."""
		raise NotImplementedError

	@abstractmethod
	def is_open(self):
		"""Returns true if mailbox is open."""
		raise NotImplementedError

	@abstractmethod
	def list_messages(self):
		"""Lists unseen messages from the mailbox for this account.
		Yields tuples (folder, message, flags) for every message.
		"""
		raise NotImplementedError

	@abstractmethod
	def request_folders(self):
		"""Returns list of folder names available in the mailbox.
		Raises an exceptions if mailbox does not support folders.
		"""
		raise NotImplementedError

	def supports_mark_as_seen(self):
		"""Returns True if mailbox supports flagging mails as seen."""
		# Default implementation
		return False

	@abstractmethod
	def mark_as_seen(self, mails):
		"""Asks mailbox to flag mails in the list as seen.
		This may raise an exception if mailbox does not support this action.
		"""
		raise NotImplementedError
		
	def supports_notifications(self):
		"""Returns True if mailbox supports notifications."""
		# Default implementation
		return False

	@abstractmethod
	def notify_next_change(self, callback=None, timeout=None):
		"""Asks mailbox to notify next change.
		Callback is called when new mail arrives or a mail is removed.
		This may raise an exception if mailbox does not support
		notifications.
		"""
		raise NotImplementedError

	@abstractmethod
	def cancel_notifications(self):
		"""Cancels notifications.
		This may raise an exception if mailbox does not support
		notifications.
		"""
		raise NotImplementedError




# Copyright 2014 - 2021 Patrick Ulbrich <zulu99@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

import os
from enum import Enum
from gi.repository import Gio

TEST_HOST = 'www.google.com'


class TestModes(Enum):
	NETWORKMONITOR	= 0
	PING			= 1


class ConnectivityTest:
	def __init__(self, testmode):
		self._testmode = testmode
		self._monitor = None
		
		
	def is_offline(self):
		if self._testmode == TestModes.NETWORKMONITOR:
			if self._monitor == None:
				# The monitor instance is based on NetworkManager if available, 
				# otherwise on the kernels netlink interface.
				self._monitor = Gio.NetworkMonitor.get_default()
			
			if self._monitor.get_connectivity() == Gio.NetworkConnectivity.FULL:
				return False
			else:
				return (not self._monitor.can_reach(Gio.NetworkAddress.new(TEST_HOST, 8080)))
		else:
			return (os.system('ping -c1 -W2 %s > /dev/null 2>&1' % TEST_HOST) != 0)




# Copyright 2012 - 2020 Patrick Ulbrich <zulu99@gmx.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

#
# This file contains constants that need to be adjusted for propper distro integration.
#

# Application version displayed in the 
# about dialog of the config window.
APP_VERSION = '2.2.0'

# The PACKAGE_NAME constant is used to configure
# 1) the path where all app data (glade files, images) is loaded from
#    (usually /usr/share/<PACKAGE_NAME>) via get_data_file() (see utils.py). 
# 2) paths for localization files generated with gen_locales
#    (usually /usr/share/locale/<LANG>/LC_MESSAGES/<PACKAGE_NAME>.mo).
# Typically, there's no need to touch this constant.
PACKAGE_NAME = 'mailnag'

# The LOCALE_DIR constant specifies the root path for localization files
# (usually you have to make it point to '/usr/share/locale').
LOCALE_DIR = '/usr/share/locale'

# The DESKTOP_FILE_DIR constant specifies the root path for .desktop files
# (usually you have to make it point to '/usr/share/applications').
DESKTOP_FILE_DIR = '/usr/share/applications'

# The LIB_DIR constant specifies the root path for the Mailnag python files
# (usually you have to make it point to <PYTHON_LIB_DIR>/Mailnag).
LIB_DIR = '/usr/lib/python3.14/site-packages/Mailnag'

# The BIN_DIR constant specifies the path for the mailnag start scripts
# (usually you have to make it point to '/usr/bin').
BIN_DIR = '/usr/bin'

# DBUS service configuration
DBUS_BUS_NAME = 'mailnag.MailnagService'
DBUS_OBJ_PATH = '/mailnag/MailnagService'
